#include "StdAfx.h"
#include <fstream>
#include "asciibmp.h"

using namespace std;

asciibmp::asciibmp()
{
	w = h = 0;
	img = 0;
	col = 0;
}

asciibmp::asciibmp(const std::string filename)
{
	w = h = 0;
	img = 0;
	col = 0;
	load(filename);
}

asciibmp::asciibmp(int width, int height)
{
	w = h = 0;
	img = 0;
	col = 0;
	create(width, height);
}

asciibmp::~asciibmp()
{
	destroy();
}

void asciibmp::destroy()
{
	if(img)
	{
		for(int y = 0; y < h; ++y)
		{
			if(img[y])
				delete [] img[y];
		}
		delete [] img;
	}
	if(col)
	{
		for(int y = 0; y < h; ++y)
		{
			if(col[y])
				delete [] col[y];
		}
		delete [] col;
	}
	w = h = 0;
}

//This function must only be called if previous memory was already released!
void asciibmp::reserveMemory()
{
	img = new char*[h];
	col = new int*[h];
	for(int i = 0; i < h; ++i)
	{
		img[i] = new char[w];
		col[i] = new int[w];
	}
}

void asciibmp::load(const string filename)
{
	//Make sure that there is nothing already loaded
	destroy();
	//Load the bitmap size
	ifstream imgFile((filename + ".txt").c_str());
	if(imgFile.fail())
		error::loadBitError(filename + ".txt");
	BITMAP *colFile = loadBitmap(filename + ".tga");
	imgFile >> w >> h;
	imgFile.ignore(1024, '\n');
	//Reserve memory
	reserveMemory();
	for(int y = 0; y < h; ++y)
	{
		for(int x = 0; x < w; ++x)
		{
			img[y][x] = imgFile.get();
			col[y][x] = getpixel(colFile, x, y);
		}
		imgFile.ignore(1024, '\n');
	}
	destroy_bitmap(colFile);
}

void asciibmp::draw(BITMAP *dest) const
{
	char myString[2] = " ";
	for(int y = 0; y < h; ++y)
	{
		for(int x = 0; x < w; ++x)
		{
			myString[0] = img[y][x];
			if(myString[0] == ' ')
				continue;
			textout_ex(dest, font, myString, x * 8, y * 8, col[y][x], -1);
		}
	}
}

void asciibmp::draw(asciibmp *dest, int dest_x, int dest_y) const
{
	for(int y = 0; y < h; ++y)
	{
		for(int x = 0; x < w; ++x)
		{
			if(col[y][x] != colors::pink)
				dest->setPixel(dest_x + x, dest_y + y, img[y][x], col[y][x]);
		}
	}
}

void asciibmp::draw(asciibmp &dest, int dest_x, int dest_y) const
{
	draw(&dest, dest_x, dest_y);
}

void asciibmp::draw(asciibmp &dest, int source_x, int source_y, int dest_x, int dest_y, int dest_w, int dest_h) const
{
	draw(&dest, source_x, source_x, dest_x, dest_y, dest_w, dest_h);
}

void asciibmp::setPixel(int x, int y, char c, int color)
{
	if(x < w && y < h && x >= 0 && y >= 0)
	{
		img[y][x] = c;
		col[y][x] = color;
	}
}

asciibmp &asciibmp::operator=(const asciibmp &ref)
{
	createFrom(ref);
	return *this;
}

void asciibmp::createFrom(const asciibmp &ref)
{
	w = ref.getW();
	h = ref.getH();
	create(w, h);
	ref.draw(this, 0, 0);
}

int asciibmp::getColor(int x, int y) const
{
	return col[y][x];
}

char asciibmp::getCharacter(int x, int y) const
{
	return img[y][x];
}

int asciibmp::getW() const
{
	return w;
}

int asciibmp::getH() const
{
	return h;
}

void asciibmp::clear(int color)
{
	for(int y = 0; y < h; ++y)
	{
		for(int x = 0; x < w; ++x)
		{
			img[y][x] = ' ';
			col[y][x] = color;
		}
	}
}

void asciibmp::create(int width, int height)
{
	destroy();
	w = width;
	h = height;
	reserveMemory();
	clear(colors::black);
}

void asciibmp::textwrite(const std::string text, int x, int y, int color)
{
	for(int i = 0; i < text.length(); ++i)
	{
		setPixel(x + i, y, text[i], color);
	}
}

void asciibmp::textwritecenter(const std::string text, int x, int y, int color)
{
	x -= text.length() >> 1;
	textwrite(text, x, y, color);
}

void asciibmp::textwriteright(const std::string text, int x, int y, int color)
{
	x -= text.length();
	textwrite(text, x, y, color);
}

void asciibmp::draw(asciibmp *dest, int source_x, int source_y, int dest_x, int dest_y, int dest_w, int dest_h) const
{
	if(w < source_x + dest_w)
		dest_w = w - source_x;
	if(dest->getW() < dest_x + dest_w)
		dest_w = dest->getW() - dest_x;
	if(h < source_y + dest_h)
		dest_h = h - source_y;
	if(dest->getH() < dest_y + dest_h)
		dest_h = dest->getH() - dest_y;
	
	if(source_x < 0)
	{
		dest_w += source_x;
		source_x = 0;
	}
	if(source_y < 0)
	{
		dest_h += source_y;
		source_y = 0;
	}
	if(dest_x < 0)
	{
		dest_w += dest_x;
		dest_x = 0;
	}
	if(dest_y < 0)
	{
		dest_h += dest_y;
		dest_y = 0;
	}
	if(dest_h <= 0)
		return;
	if(dest_w <= 0)
		return;
	
	for(int y = 0; y < dest_h; ++y)
	{
		for(int x = 0; x < dest_w; ++x)
		{
			if(col[y + source_y][x + source_x] != colors::pink)
				dest->setPixel(dest_x + x, dest_y + y, img[y + source_y][x + source_x], col[y + source_y][x + source_x]);
		}
	}
}

void asciibmp::hline(char character, int x1, int y, int x2, int color)
{
	for(int x = x1; x <= x2; ++x)
	{
		setPixel(x, y, character, color);
	}
}

void asciibmp::vline(char character, int x, int y1, int y2, int color)
{
	for(int y = y1; y <= y2; ++y)
	{
		setPixel(x, y, character, color);
	}
}
